﻿namespace university_ok
{
    public class NEPTUNSystem
    {
        // Ezek az adatok tipikusan adatbázisban lennének élesben
        private List<Student> students = new List<Student>();
        private List<Semester> semesters = new List<Semester>();
        private List<Subject> subjects = new List<Subject>();
        private List<FinishedSubject> finishedSubjects = new List<FinishedSubject>();

        public NEPTUNSystem(List<Student> students, List<Semester> semesters)
        {
            this.students = students;
            this.semesters = semesters;

            // Minden hallgatót "regisztrálunk" a NEPTUN-ba
            foreach (var st in this.students)
            {
                st.System = this;
                Console.WriteLine("###NEPTUN### Registered student " + st.Name);
            }

            // Minden tárgyat minden szemeszterben "regisztrálunk" a NEPTUN-ba
            foreach (var se in this.semesters)
            {
                foreach (var su in se.Subjects)
                {
                    this.subjects.Add(su);
                    Console.WriteLine("###NEPTUN### Registered subject " + su.Name);
                }
            }
        }

        // A bool visszatérési érték azért van, hogy ha a hallgató már megcsinálta az adott szemesztert, nem veheti fel megint
        public bool StartActiveSemester(Student st, Semester se)
        {
            if (!students.Contains(st) || !semesters.Contains(se))
                throw new Exception("Invalid student or semester!");

            // Ha a hallgató minden tárgyat megcsinált, nem veheti fel még egyszer a félévet
            if (IsStudentFinishedSemester(st, se))
            {
                Console.WriteLine("###NEPTUN### " + st.Name + " already finished semester, cannot start it again");
                return false;
            }

            // Minden, még nem teljesített szemeszterbeli tárgyhoz hozzárendeljük a hallgatót
            foreach (var su in se.Subjects)
            {
                if (!IsStudentFinishedSubject(st, su))
                {
                    su.AddActiveStudent(st);
                }
                else
                {
                    Console.WriteLine("###NEPTUN### " + st.Name + " already finished subject " + su.Name);
                }
            }

            return true;
        }

        public void EndActiveSemester(Student st, Semester se)
        {
            if (!students.Contains(st) || !semesters.Contains(se))
                throw new Exception("Invalid student or semester!");

            // Minden, szemeszterbeli tárgyról levesszük a hallgatót
            foreach (var su in se.Subjects)
            {
                // Amennyiben a hallgató elvégezte a tárgyat, regisztráljuk a NEPTUN-ban
                if (su.IsSubjectFinished(st))
                {
                    Console.WriteLine("###NEPTUN### " + st.Name + " finished subject " + su.Name);
                    finishedSubjects.Add(new FinishedSubject() { student = st, subject = su });
                }
                    
                su.RemoveActiveStudent(st);
            }
        }

        public bool RequestDegree(Student st)
        {
            Console.WriteLine("###NEPTUN### " + st.Name + " requested degree");

            // Ha nem teljesített minden tárgyat, nem kaphat diplomát (őrfeltétel)
            if (!IsStudentEligibleForDegree(st))
            {
                return false;
            }

            // Ha megkapta a diplomáját, a hallgatót eltávolítjuk a NEPTUN rendszerből
            if (students.Contains(st))
                students.Remove(st);

            Console.WriteLine("###NEPTUN### " + st.Name + " got their degree ");

            return true;
        }

        public bool IsStudentEligibleForDegree(Student st)
        {
            // Először elkérjük a hallgató teljesített tárgyait
            var finished = finishedSubjects.Where(fs => fs.student == st).ToList();

            // Majd megnézzük, hogy van-e olyan tárgy, amit nem teljesített
            return subjects.All(s => finished.Any(f => f.subject == s));
        }

        public bool IsStudentFinishedSubject(Student st, Subject su)
        {
            return finishedSubjects.Any(fs => fs.student == st && fs.subject == su);
        }

        public bool IsStudentFinishedSemester(Student st, Semester se)
        {
            return se.Subjects.All(su => IsStudentFinishedSubject(st, su));
        }
    }
}
